@@ -17,7 +17,9 @@ from sqlalchemy.schema import (ForeignKeyConstraint,
                                Index)
 
 from migrate import exceptions
+import sqlalchemy.sql.compiler
 from migrate.changeset import constraint
+from migrate.changeset import util
 
 from sqlalchemy.schema import AddConstraint, DropConstraint
 from sqlalchemy.sql.compiler import DDLCompiler
@@ -55,7 +57,7 @@ class AlterTableVisitor(SchemaVisitor):
             # adapt to 0.6 which uses a string-returning
             # object
             self.append(" %s" % ret)
-            
+
     def _to_table(self, param):
         """Returns the table object for the given param object."""
         if isinstance(param, (sa.Column, sa.Index, sa.schema.Constraint)):
@@ -112,7 +114,7 @@ class ANSIColumnGenerator(AlterTableVisitor, SchemaGenerator):
         # SA bounds FK constraints to table, add manually
         for fk in column.foreign_keys:
             self.add_foreignkey(fk.constraint)
-        
+
         # add primary key constraint if needed
         if column.primary_key_name:
             cons = constraint.PrimaryKeyConstraint(column,
@@ -156,8 +158,8 @@ class ANSISchemaChanger(AlterTableVisitor, SchemaGenerator):
     def visit_table(self, table):
         """Rename a table. Other ops aren't supported."""
         self.start_alter_table(table)
-        self.append("RENAME TO %s" % self.preparer.quote(table.new_name,
-                                                         table.quote))
+        q = util.safe_quote(table)
+        self.append("RENAME TO %s" % self.preparer.quote(table.new_name, q))
         self.execute()
 
     def visit_index(self, index):
@@ -171,8 +173,8 @@ class ANSISchemaChanger(AlterTableVisitor, SchemaGenerator):
                     self.preparer.quote(
                         self._validate_identifier(
                             index.new_name, True), index.quote)))
-        else:
-            # SA >= 0.6.5
+        elif hasattr(self, '_index_identifier'):
+            # SA >= 0.6.5, < 0.8
             self.append("ALTER INDEX %s RENAME TO %s" % (
                     self.preparer.quote(
                         self._index_identifier(
@@ -180,6 +182,23 @@ class ANSISchemaChanger(AlterTableVisitor, SchemaGenerator):
                     self.preparer.quote(
                         self._index_identifier(
                             index.new_name), index.quote)))
+        else:
+            # SA >= 0.8
+            class NewName(object):
+                """Map obj.name -> obj.new_name"""
+                def __init__(self, index):
+                    self.name = index.new_name
+                    self._obj = index
+
+                def __getattr__(self, attr):
+                    if attr == 'name':
+                        return getattr(self, attr)
+                    return getattr(self._obj, attr)
+
+            self.append("ALTER INDEX %s RENAME TO %s" % (
+                    self._prepared_index_name(index),
+                    self._prepared_index_name(NewName(index))))
+
         self.execute()
 
     def visit_column(self, delta):
@@ -209,7 +228,8 @@ class ANSISchemaChanger(AlterTableVisitor, SchemaGenerator):
     def start_alter_column(self, table, col_name):
         """Starts ALTER COLUMN"""
         self.start_alter_table(table)
-        self.append("ALTER COLUMN %s " % self.preparer.quote(col_name, table.quote))
+        q = util.safe_quote(table)
+        self.append("ALTER COLUMN %s " % self.preparer.quote(col_name, q))
 
     def _visit_column_nullable(self, table, column, delta):
         nullable = delta['nullable']
@@ -232,7 +252,8 @@ class ANSISchemaChanger(AlterTableVisitor, SchemaGenerator):
 
     def _visit_column_name(self, table, column, delta):
         self.start_alter_table(table)
-        col_name = self.preparer.quote(delta.current_name, table.quote)
+        q = util.safe_quote(table)
+        col_name = self.preparer.quote(delta.current_name, q)
         new_name = self.preparer.format_column(delta.result_column)
         self.append('RENAME COLUMN %s TO %s' % (col_name, new_name))
 
@@ -257,7 +278,7 @@ class ANSIConstraintCommon(AlterTableVisitor):
             ret = cons.name
         else:
             ret = cons.name = cons.autoname()
-        return self.preparer.quote(ret, cons.quote)
+        return ret
 
     def visit_migrate_primary_key_constraint(self, *p, **k):
         self._visit_constraint(*p, **k)
